@@ -15,9 +15,13 @@
 # with this program. If not, see <http://www.gnu.org/licenses/>.
 """cubicweb-vcreview entity's classes"""
 
+from random import choice
+
 from logilab.common.decorators import monkeypatch
 
 from cubicweb.entities import AnyEntity
+from cubicweb.view import EntityAdapter
+from cubicweb.selectors import is_instance
 
 from cubes.comment import entities as comment
 from cubes.vcsfile import entities as vcsfile
@@ -88,6 +92,42 @@ class CommentITreeAdapter(comment.CommentITreeAdapter):
         return path
 
 
+class PatchReviewBehaviour(EntityAdapter):
+    __regid__ = 'IPatchReviewControl'
+    __select__ = is_instance('Patch')
+
+    def reviewers_rset(self):
+        return self._cw.execute(
+            'Any U WHERE EXISTS(U in_group G, G name "reviewers") '
+            'OR EXISTS(P patch_repository R, R repository_reviewer U, P eid %(p)s)',
+            {'p': self.entity.eid})
+
+    def set_reviewers(self, transition):
+        if self.entity.patch_reviewer:
+            return
+        # XXX only consider review affected during the last week by getting date
+        # of the first in-progress -> pending-review transition? or use
+        # something like a review_started_on attribute on patches ?
+        nbpatches = dict(self._cw.execute(
+            'Any U, COUNT(P) GROUPBY U WHERE P patch_reviewer U, '
+            'P in_state S, S name IN ("in-progress", "pending-review")'))
+        ueids = []
+        mincount = None
+        for ueid, in self.reviewers_rset():
+            count = nbpatches.get(ueid, 0)
+            if mincount is None:
+                mincount = count
+                ueids = [ueid]
+            elif count < mincount:
+                mincount = count
+                ueids = [ueid]
+            elif count == mincount:
+                ueids.append(ueid)
+        if ueids:
+            self._cw.execute('SET P patch_reviewer U WHERE P eid %(p)s, U eid %(u)s',
+                             {'p': self.entity.eid, 'u': choice(ueids)})
+
+
 def registration_callback(vreg):
     vreg.register_all(globals().values(), __name__, (CommentITreeAdapter,))
     vreg.register_and_replace(CommentITreeAdapter, comment.CommentITreeAdapter)
