@@ -20,7 +20,9 @@ __docformat__ = "restructuredtext en"
 import re
 
 from cubicweb.server import hook
-from cubicweb.selectors import is_instance
+from cubicweb.selectors import is_instance, on_fire_transition
+from cubicweb.sobjects import notification
+from cubicweb.hooks import notification as notificationhooks
 
 IGNORE_FILES = set( ('README', 'series', 'status', '.hgignore', '.hgtags') )
 
@@ -141,7 +143,7 @@ class SearchPatchStateInstructionOp(hook.DataOperationMixIn,
                     self.ensure_wont_be_deleted(patch)
                 elif words[0] == 'applied' and words[1] in rejectcandidates:
                     patch = rejectcandidates.pop(words[1])
-                    fire(patch, 'accept', u'accepted by %s' % rev.author)
+                    fire(patch, 'apply', u'applied by %s' % rev.author)
                     self.ensure_wont_be_deleted(patch)
                     # XXX search source repository changeset id
                     #patch.set_relations(applied_at=rev)
@@ -180,6 +182,66 @@ class MarkPatchAsDeletedOp(hook.DataOperationMixIn, hook.SingleLastOperation):
                 patch.cw_adapt_to('IWorkflowable').fire_transition('file deleted')
 
 
+class SetReviewerOp(hook.DataOperationMixIn, hook.Operation):
+
+    def precommit_event(self):
+        for trinfo in self.get_data():
+            patch = trinfo.for_entity
+            patch.cw_adapt_to('IPatchReviewControl').set_reviewers(trinfo.transition)
+
+class SetReviewerHook(hook.Hook):
+    """register an operation to set a reviewer when the 'ask review' transition
+    is fired
+    """
+    __regid__ = 'vcreview.set-reviewer'
+    __select__ = hook.Hook.__select__ & on_fire_transition('Patch', 'ask review')
+    events = ('after_add_entity',)
+
+    def __call__(self):
+        SetReviewerOp.get_instance(self._cw).add_data(self.entity)
+
+
+class ReviewerSetHook(hook.Hook):
+    """send a notification when a user is assigned to review a patch and ensure
+    he is in the patch nosy-list
+    """
+    __regid__ = 'vcreview.reviewer-set'
+    __select__ = hook.Hook.__select__ & hook.match_rtype('patch_reviewer')
+    events = ('after_add_relation',)
+
+    def __call__(self):
+        # send notification
+        rset = self._cw.eid_rset(self.eidfrom)
+        view = self._cw.vreg['views'].select(
+            'vcreview.notifications.reviewer-set', self._cw,
+            rset=rset, user=self._cw.eid_rset(self.eidto).get_entity(0, 0))
+        notificationhooks.RenderAndSendNotificationView(self._cw, view=view)
+        # ensure user is in the nosy-list
+        self._cw.execute('SET P nosy_list U WHERE P eid %(p)s, U eid %(u)s, NOT P nosy_list U',
+                         {'p': self.eidfrom, 'u': self.eidto})
+
+class ReviewerSetNotificationView(notification.NotificationView):
+    __regid__ = 'vcreview.notifications.reviewer-set'
+    __select__ = is_instance('Patch')
+    content = _('''
+You have been assigned for review of patch %(title)s
+
+url: %(url)s
+''')
+    def recipients(self):
+        reviewer = self.cw_extra_kwargs['user']
+        email = reviewer.cw_adapt_to('IEmailable').get_email()
+        if email:
+            return [(email, reviewer.property_value('ui.language'))]
+        return []
+
+    def subject(self):
+        patch = self.cw_rset.get_entity(0, 0)
+        return self._cw._('[%(repo)s] patch %(patch)s has been assigned to you') % {
+            'app': self._cw.vreg.config.appid,
+            'repo': patch.repository.dc_title(),
+            'patch': patch.dc_title()}
+
 from cubes.nosylist import hooks as nosylist
 nosylist.S_RELS |= set(('has_activity', 'patch_revision'))
 # XXX should add 'comments' relation in set below
