@@ -52,7 +52,8 @@ class PatchCreationHooksTC(VCSRepositoryTC):
     def _create_repo(self):
         vcsrepo = super(PatchCreationHooksTC, self)._create_repo()
         # circular relations, only to make it think it is a patch repository
-        vcsrepo.set_relations(has_patch_repository=vcsrepo)
+        vcsrepo.set_relations(has_patch_repository=vcsrepo,
+                              repository_reviewer=self.user())
         return vcsrepo
 
     def test_imported_patches(self):
@@ -98,7 +99,7 @@ class PatchCreationHooksTC(VCSRepositoryTC):
         self.assertEqual(revnums(patch3_2), [7, 10, 15])
 
         self.assertTrue(patch3_2.nosy_list)
-        self.assertEqual(len(MAILBOX), 2)
+        self.assertEqual(len(MAILBOX), 3)
         self.assertEqual(MAILBOX[0].subject, '[%s] patch %s is now in state "pending-review"'
                          % (patch3_2.repository.dc_title(), patch3_2.dc_title()))
         self.assertEqual(MAILBOX[0].recipients, ['admin@cwo'])
@@ -110,10 +111,18 @@ review asked by Sylvain Th\xe9nault <sylvain.thenault@logilab.fr> in 9cf50f15d9a
 
 url: http://testing.fr/cubicweb/patch/%s\n''' % patch3_2.eid)
 
-        self.assertEqual(MAILBOX[1].subject, '[%s] patch %s is now in state "rejected"'
+        self.assertEqual(MAILBOX[1].subject, '[%s] patch %s has been assigned to you'
                          % (patch3_2.repository.dc_title(), patch3_2.dc_title()))
         self.assertEqual(MAILBOX[1].recipients, ['admin@cwo'])
         self.assertMultiLineEqual(MAILBOX[1].content, u'''
+You have been assigned for review of patch patch3.diff
+
+url: http://testing.fr/cubicweb/patch/%s\n''' % patch3_2.eid)
+
+        self.assertEqual(MAILBOX[2].subject, '[%s] patch %s is now in state "rejected"'
+                         % (patch3_2.repository.dc_title(), patch3_2.dc_title()))
+        self.assertEqual(MAILBOX[2].recipients, ['admin@cwo'])
+        self.assertMultiLineEqual(MAILBOX[2].content, u'''
 cubicweb changed status from <pending-review> to <rejected> for entity
 'patch3.diff'
 
@@ -160,7 +169,7 @@ url: http://testing.fr/cubicweb/task/%s\n''' % done.eid)
         op = hooks.SearchPatchStateInstructionOp.get_instance(self.session)
         op.add_data(revision)
         self.commit() # will process our fake operation
-        patch3_2.clear_all_caches()
+        patch3_2.cw_clear_all_caches()
         self.assertEqual(patch3_2.cw_adapt_to('IWorkflowable').state, 'applied')
         #self.assertEqual(patch3_2.applied_at[0].eid, patch3_2.revisions[-1].rev.eid) XXX
 
@@ -172,7 +181,7 @@ url: http://testing.fr/cubicweb/task/%s\n''' % done.eid)
 admin changed status from <pending-review> to <applied> for entity
 'patch3.diff'
 
-accepted by syt
+applied by syt
 
 remaining tasks:
 
