@@ -13,12 +13,14 @@
 #
 # You should have received a copy of the GNU Lesser General Public License along
 # with this program. If not, see <http://www.gnu.org/licenses/>.
-"""cubicweb-vcreview primary views adapters for web ui"""
+"""cubicweb-vcreview primary views and adapters for the web ui"""
+
 __docformat__ = "restructuredtext en"
 _ = unicode
 
 from logilab.mtconverter import xml_escape
 
+from cubicweb import tags
 from cubicweb.selectors import score_entity, is_instance
 from cubicweb.view import EntityView
 from cubicweb.schema import display_name
@@ -37,6 +39,7 @@ _afs = uicfg.autoform_section
 
 _pvs.tag_subject_of(('Patch', 'patch_repository', '*'), 'hidden') # in breadcrumbs
 _pvs.tag_subject_of(('Patch', 'patch_revision', '*'), 'hidden') # table in attributes
+_pvs.tag_subject_of(('Patch', 'patch_reviewer', '*'), 'attributes')
 _pvdc.tag_subject_of(('Patch', 'applied_at', '*'), {'vid': 'incontext'})
 
 class PatchPrimaryView(primary.PrimaryView):
@@ -51,11 +54,43 @@ class PatchPrimaryView(primary.PrimaryView):
         super(PatchPrimaryView, self).render_entity_attributes(entity)
         self.w(u'<h4>%s</h4>' % self._cw._('Patch revisions'))
         rset = self._cw.execute(
-            'Any VC,RA,RB,RC,RD,COUNT(C) GROUPBY VC,RA,RB,RC,RD ORDERBY RC DESC '
+            'Any VC,RA,RB,RC,RD,VC, R ORDERBY RC DESC '
             'WHERE X eid %(x)s, X patch_revision VC, VC from_revision R,'
-            'R author RA, R branch RB, R creation_date RC, R description RD, '
-            'IP? point_of VC, C? comments IP', {'x': entity.eid})
-        self.wview('table', rset)
+            'R author RA, R branch RB, R creation_date RC, R description RD',
+            {'x': entity.eid})
+        _, __ = self._cw._, self._cw.__
+        self.wview('table', rset,
+                   headers=[__('Revision'), __('author'), __('branch'),
+                            __('creation_date'), __('commit message'),
+                            _('nb comments')],
+                   cellvids={0: 'vcreview.fileversion',
+                             5: 'vcreview.revcommentcount'})
+
+class DVCFileVersion(EntityView):
+    __regid__ = 'vcreview.fileversion'
+    __select__ = is_instance('DeletedVersionContent', 'VersionContent')
+
+    def entity_call(self, entity):
+        self.w(tags.a(self._cw._(u'%(file)s (revision %(revnum)s)') % {
+            'file':entity.dc_title(), 'revnum': entity.revision},
+                      href=entity.absolute_url()))
+
+class DVCCommentCount(EntityView):
+    __regid__ = 'vcreview.revcommentcount'
+    __select__ = is_instance('DeletedVersionContent')
+
+    def cell_call(self, row, col):
+        self.w(u'&#160;')
+
+class VCCommentCount(EntityView):
+    __regid__ = 'vcreview.revcommentcount'
+    __select__ = is_instance('VersionContent')
+
+    def cell_call(self, row, col):
+        count = self._cw.execute(
+            'Any COUNT(C) WHERE IP point_of VC, C comments IP, VC eid %(x)s',
+            {'x': self.cw_rset[row][col]})[0][0]
+        self.w(unicode(count))
 
 
 # version content primary view #################################################
@@ -90,6 +125,8 @@ _afs.tag_attribute(('VersionContent', 'data_encoding'), 'main', 'attributes')
 _abaa.tag_subject_of(('Repository', 'has_patch_repository', '*'), True)
 _abaa.tag_object_of(('*', 'patch_repository', 'Repository'), False)
 _pvs.tag_object_of(('*', 'patch_repository', 'Repository'), 'hidden')
+_pvs.tag_subject_of(('Repository', 'repository_committer', '*'), 'attributes')
+_pvs.tag_subject_of(('Repository', 'repository_reviewer', '*'), 'attributes')
 
 
 class RepositoryPatchesTab(EntityView):
@@ -109,17 +146,18 @@ class RepositoryPatchesTable(EntityView):
     __regid__ = 'vcreview.repository.patches'
     __select__ = is_instance('Repository') & score_entity(lambda x: x.patchrepo_of)
 
+    rql = ('Any P,PO,PB,PS,COUNT(TR) GROUPBY P,PO,PB,PS WHERE '
+           'P originator PO, P branch PB, P in_state PS, TR? wf_info_for P, '
+           'P patch_repository R, R eid %(x)s')
+
     def entity_call(self, entity):
         linktitle = self._cw._('Patches for %s') % entity.dc_title()
-        rql = ('Any P,PO,PB,PS,COUNT(TR) GROUPBY P,PO,PB,PS WHERE '
-               'P originator PO, P branch PB, P in_state PS, TR? wf_info_for P, '
-               'P patch_repository R, R eid %(x)s')
-        linkurl = self._cw.build_url(rql=rql % {'x': entity.eid},
+        linkurl = self._cw.build_url(rql=self.rql % {'x': entity.eid},
                                      vtitle=linktitle)
         self.w('<p>%s. <a href="%s">%s</a></p>' % (
             self._cw._('Table below only show active patches'),
             xml_escape(linkurl), self._cw._('Show all patches')))
-        rql += ', PS name IN ("in-progress", "pending-review", "deleted")'
+        rql = self.rql + ', PS name IN ("in-progress", "pending-review", "deleted")'
         rset = self._cw.execute(rql, {'x': entity.eid})
         self.wview('table', rset, 'noresult', paginate=True, displayfilter=True)
 
