@@ -28,14 +28,14 @@ class AllActivePatches(StartupView):
     __regid__ = 'vcreview.allactivepatches'
     title = _('All active patches')
 
+    rql = ('Any P,PO,PB,PS,COUNT(TR),R GROUPBY R,P,PO,PB,PS '
+           'ORDERBY RT,PB,PO WHERE P originator PO, P branch PB, P in_state PS,'
+           'TR? wf_info_for P, P patch_repository R, R title RT, '
+           'PS name IN ("in-progress", "pending-review", "deleted")')
+
     @cached
     def filter_box_context_info(self):
-        rset = self._cw.execute(
-            'Any P,PO,PB,PS,COUNT(TR),R GROUPBY R,P,PO,PB,PS '
-            'ORDERBY RT,PB,PO WHERE P originator PO, P branch PB, P in_state PS,'
-            'TR? wf_info_for P, P patch_repository R, R title RT, '
-            'PS name IN ("in-progress", "pending-review", "deleted")'
-            )
+        rset = self._cw.execute(self.rql)
         return rset, 'vcreview.patches.table', domid(self.__regid__), False
 
     def call(self):
@@ -44,10 +44,27 @@ class AllActivePatches(StartupView):
         self.wview(vid, rset, 'noresult', divid=divid)
 
 
-class AllActivePatchesTable(tableview.TableView):
+class UserWorkList(AllActivePatches):
+    __regid__ = 'vcreview.patches.worklist'
+    title = _('My review worklist')
+
+    rql = ('Any P,PO,PB,PS,COUNT(TR),R GROUPBY R,P,PO,PB,PS '
+           'ORDERBY RT,PB,PO WHERE P originator PO, P branch PB, P in_state PS,'
+           'TR? wf_info_for P, P patch_repository R, R title RT, U eid %(u)s, '
+           'EXISTS(PS name "pending-review" AND P patch_reviewer U) '
+           'OR EXISTS(PS name IN ("reviewed", "deleted") AND EXISTS(R repository_committer U) '
+           'OR EXISTS(U in_group G, G name "commiters"))')
+
+    @cached
+    def filter_box_context_info(self):
+        rset = self._cw.execute(self.rql, {'u': self._cw.user.eid})
+        return rset, 'vcreview.patches.table', domid(self.__regid__), False
+
+
+
+class PatchesTable(tableview.TableView):
     __regid__ = 'vcreview.patches.table'
     # XXX selector
     def call(self, **kwargs):
         kwargs['paginate'] = True
-        super(AllActivePatchesTable, self).call(**kwargs)
-
+        super(PatchesTable, self).call(**kwargs)
